#-*-coding:utf-8-*-

"""
fournit la classe FiltreFixe selon la littérature concernant le
traitrement du signal. 
voir p. ex. Tisserand p. 180
        
En laisssant, dans passe_haut,  '1.0 - 2*a_inf', on n'obtient pas
'ho' comme indiqué dans Tisserand p. 183.
mais auditivement c'est ce qu'il faut ...        
"""
    
import numpy as NY
from samplerate import sr
from conversions08 import extremum, prony2freq


__author__ = "Copyright (c) 2007-09 René Bastian (Wissembourg, FRANCE)"
__date__ = "2008.09.21, 09.02.03"

def construire_coeffs(a, centre):
    """
    construit le vecteur des coefficients
    """
    return NY.concatenate((a[::-1], NY.array([centre]), a))

class FiltreFixe(object):
    """
    une classe de filtres à fréquence de coupure fixes
    (passe-bas, passe-haut, passe-bande, coupe-bande)
    """
    def __init__(self, genre, nc):
        """
        'nc' : nombre de coefficients
        'genre' : "pb", "lp" -> passe-bas
                  "ph", "hp" -> passe-haut
                  "xb" -> coupe-bande
                  "bp" -> passe-bande
        Le nombre final de coefficients est 'nc*2 + 1'.
        le fenêtrage peut se faire par un profil.
        """
        self.genre = genre
        self.nc = nc
        self.coeffs = None

    def __str__(self):
        "pour avoir les données sous forme de string"
        return self.__class__.__name__  + str(self.__dict__)

    def affiche(self):
        "pour afficher"
        print "FiltreFixe", self.genre
        print "nc", self.nc, "->", 2*self.nc+1, "coeffs"
        if self.coeffs != None:
            print self.coeffs
        else:
            print "Il n'y a pas encore de coeffs."

    def sinc(self, a):
        "calcule les coefficients de base"
        K = NY.arange(1., self.nc+1, 1.)*2*NY.pi*a
        g = 2*a*NY.sin(K) / K
        return g
    
    def _coeffsPasseBas(self, freq):
        """
        n = nombre de coefficients "a" à calculer
        fc= fréquence de coupure    
        """
        a_sup = freq / sr
        x = self.sinc(a_sup)
        self.coeffs = construire_coeffs(x, 2*a_sup)
        return self.coeffs
    
    def _coeffsPasseHaut(self, fc):
        """
        n = nombre de coefficients "a" à calculer
        fc= fréquence de coupure    
        """
        a_inf = fc / sr
        x = -self.sinc(a_inf)
        
        
        self.coeffs = construire_coeffs(x, 1.0 - 2*a_inf)
        return self.coeffs 

    def _coeffsPasseBande(self, fcinf, fcsup):
        """
        n = nombre de coefficients "a" à calculer
        fcxxx= fréquences de coupure    
        """
        a_inf = fcinf / sr
        a_sup = fcsup / sr
        x = self.sinc(a_sup) - self.sinc(a_inf)
        
        
        self.coeffs = construire_coeffs(x, a_sup - a_inf)
        return self.coeffs 
    
    def _coeffsCoupeBande(self, fcinf, fcsup):
        """
        n = nombre de coefficients "a" à calculer
        fcxx= fréquences de coupure    
        """
        a_inf = fcinf / sr
        a_sup = fcsup / sr
        x = self.sinc(a_inf) - self.sinc(a_sup)
        
        self.coeffs = construire_coeffs(x, a_inf - a_sup)
        return self.coeffs
    
    def __call__(self, signal, prony=None, nc=None):
        """
        effectue le filtrage du signal
        """
        if extremum(signal) == 0.0:
            return signal
        try:
            freq = prony2freq(prony)
        except ValueError:
            pronyinf, pronysup = prony
            freqinf = prony2freq(pronyinf)
            freqsup = prony2freq(pronysup)
        if nc != None:
            self.nc = nc
        if prony != None:
            if self.genre in ["pb", "lp"]:
                self._coeffsPasseBas(freq)
            elif self.genre in ["ph", "hp"]:
                self._coeffsPasseHaut(freq)
            elif self.genre in ["xb"]:
                self._coeffsCoupeBande(freqinf, freqsup)
            elif self.genre in ["bp"]:
                self._coeffsPasseBande(freqinf, freqsup)

        w = NY.convolve(signal, self.coeffs)
        
        if extremum(w) == 0.0:
            print "FiltreFixe: le resultat est un vecteur nul"
            print "coeffs", self.coeffs
            print "min, max, extremum", w.min(), w.max(), extremum(w)
            return w
        else:
            return w / extremum(w)


