#-*-coding:utf-8-*-

"""
fournit la classe Moyenneur pour un filtre fixe selon le chap. 15 de
'The Scientist and Engineer's Guide to Digital Signal Porcessing'
de Steven W. Smith.
"""

__author__ = "Copyright (c) 2005 René Bastian (Wissembourg, FRANCE)"
__date__ = "2006.01.01"


import numpy as NY
from conversions08 import prony2samples
import ajuster 


class Moyenneur:
    """
    essai pour implémenter les 4 genres de filtres en moyenneur
    y[i] = (x[i] - x[i-p-1])/M + y[i-1] (1)    
    Il n'est pas nécessaire de faire une classe, mais on cela verra
    plus tard.
    """
    def __init__(self):
        """
        crée une instance de la classe
        """
        pass
    
    def __call__(self, signal, p):
        """
        'p' : nombre d'échantillons à moyenner
        Chaque échantillon du signal sortant est la moyenne arithmétique
        de 'p' échantillons consécutifs du signal entrant.        
        """
        y = NY.zeros(len(signal) + p, NY.float64)
        signal = NY.concatenate((NY.zeros(p, NY.float64), signal))
        for i in range(p, len(signal)):
            y[i] = signal[i] - signal[i-p] + y[i-1]
        return y[p:]

    def filtre(self, signal, p):
        """comme __call__"""
        return self.__call__(signal, p)

    def filtreV(self, signal, vp):
        """
        application d'une suite à un signal
        la suite est en général plus courte que le signal donc
        on calcule une suite allongée.
        """
        pmax = max(vp)
        y = NY.zeros(len(signal) + pmax, NY.float64)
        signal = NY.concatenate((NY.zeros(pmax, NY.float64), signal))
        
        if len(vp) < len(signal):
            vp = ajuster.allonger(vp, len(signal))
        for i in range(pmax, len(signal)):
            p = vp[i-pmax]            
            y[i] = (signal[i] - signal[i-p])/p + y[i-1]
        return y[pmax:]



