#-*-coding:utf-8-*-

"""timbres de flûte avec portamento"""


__author__ = "Copyright (c) 2005 René Bastian (Wissembourg, FRANCE)"
__date__ = "2006.08.17"


class FloeteMono:
    """
    semble correct de 36.0 à 100.0
    bien à partir de 55.0
    """
    def __init__(self,
                 Flute = [[_p.cubica, 1./5, 0.0, 0.0, 1.0, 0.0],
                          [_p.cubica, 1./4, 1.0, 0.0, -1.0, 0.0],
                          [_p.cubica, 1./3, -1.0, 0.0, 0.0, 0.0]],
                 couple = (75663,12983)):
                 
        """
        timbre de flûte sans panorama
        """
        self.g = Guirlande()        
        self.table_flute = _p.ProfilBase(Flute).temps(1.0)
        self.alea = serie.Serie002(*couple)
        
    def __call__(self, Enveloppe, *ListeParams):
        """
        'Enveloppe : enveloppe finale du son 
        'ListeParams : duree, dt, ListeTons
        'duree : durée d'une note
        'dt : durée de l'intervalle de temps entre deux notes
        'ListeTons : liste de tons
        """
        canalg = CanalMono()        
        Env = _p.ProfilBase(Enveloppe)        
        mf = NA.array([])
        for params in ListeParams:            
            duree, dt, ListeTons = params
            mf = NA.concatenate((mf, self.g.traitL(duree, dt, ListeTons)))
        w = _o.ondeTableMF(self.table_flute, mf)
        n = len(w)
        env = Env.samples(n)
        w, env = _a.prolongerN(w, env)
        w = w * env
        return w
        
    def ton(self, duree, Pronyvar, Enveloppe, Panorama):
        """
        'Enveloppe : enveloppe finale du son
        'Panorama : panorama final du son
        'Pronyvar : profil de variation de la hauteur Prony
        
        """        
        mf = _u.prony2freq(_p.ProfilBase(Pronyvar).temps(duree))
        w = _o.ondeTableMF(self.table_flute, mf)
        n = len(w)
        env = _p.ProfilBase(Enveloppe).samples(n)
        
        w = w * env
        n = len(w)+1
        z = NA.zeros(n, NA.Float64)
        for i in range(1, n):
            z[i] = (w[i-1]+w[i-2])/2
        pano = _p.ProfilBase(Panorama).samples(n)
        z, pano = _a.prolongerN(z, pano)
        wg = z * pano
        wd = z * (1.0 - pano)
        return wg, wd

    def multiple(self, nsons, epaisseur, Enveloppe, *ListeParams):
        c = CanalMono()
        Env = _p.ProfilBase(Enveloppe)        
        mf = NA.array([])
        for params in ListeParams:            
            duree, dt, ListeTons = params
            mf = NA.concatenate((mf, self.g.traitL(duree, dt, ListeTons)))
        nmf = len(mf)
        for i in range(nsons):
            t0 = self.alea.uniform(0., 1.)
            y0 = self.alea.uniform(0., epaisseur)
            y1 = self.alea.uniform(0., epaisseur)
            y2 = self.alea.uniform(0., epaisseur)
            p = [[_p.recta, t0, y0, y1],
                 [_p.recta, 1.-t0, y1, y2]]
            v = _p.ProfilBase(p).samples(nmf)
            mfalea = mf * (1. + v)
            w = _o.ondeTableMF(self.table_flute, mfalea)
            n = len(w)
            env = Env.samples(n)
            w, env = _a.prolongerN(w, env)
            w = w * env
            c.addT(0.0, w)
        return c.ex()

    def grenu(self, nsons, epaisseur, Enveloppe, *ListeParams):
        """
        n'est pas satisfaisant car le bruit blanc est
        superposé - l'autre essai (multiplier la mf par
        un profil sériel) n'est pas fameux non plus
        """
        c = CanalMono()
        Env = _p.ProfilBase(Enveloppe)        
        mf = NA.array([])
        for params in ListeParams:            
            duree, dt, ListeTons = params
            mf = NA.concatenate((mf, self.g.traitL(duree, dt, ListeTons)))
        nmf = len(mf)
        w0 = _o.ondeTableMF(self.table_flute, mf)
        for i in range(nsons):            
            
            v = self.alea.uniform(0., epaisseur, len(w0))
            w = w0 * (1.+v)
            n = len(w)
            env = Env.samples(n)
            w, env = _a.prolongerN(w, env)
            w = w * env
            c.addT(0.0, w)
        return c.ex()


