#-*-coding:utf-8-*-

import serie
import profils as _p
import canal


class ProfilCrecelle:
    "un profil percussif"
    def __init__(self, laps1, laps2, couple=(756412, 91275476)):
        """
        'laps1, laps2 : microdurées (0.002, 0.003)
        """
        self.alea = serie.Serie002(*couple)
        self.laps1 = laps1
        self.laps2 = laps2
        
    def chlaps(self, laps1, laps2):
        """
        change les laps
        """
        self.laps1 = laps1
        self.laps2 = laps2
        
    def __call__(self, duree, maximum):
        """
        'duree' : duree de la suite de bruits
        'maximum' : maximum du profil
        """
        D = _p.suiteDurees(duree, self.laps1, self.laps2, self.alea)
        P = []
        n=len(D)
        v0 = maximum
        v1 = maximum*0.5
        for i in range(n):
            
            P.append([_p.recta, D[i], v0, v1])
            v0 = v1
            if i%2 == 0:
                v1 = v1*1.25
            else:
                v1 = v1*0.5
            if v1 == 0.0:
                break
        try:
            w = _p.profil(P)
        except: 
            print "Erreur:ProfilCrecelle:__call__"
            print P
            return
        w = self.alea.uniform(-1., 1., len(w))*w
        return w

    def rafaleidentique(self, n, duree, maximum,
                        ecartmin=0.01, ecartmax=0.15):
        """
        on produit un bruit w et celui-ci
        est reproduit n fois sur la durée duree
        (produit une rafale faite d'impulsions identiques)
        """
        pt = 0.0
        kanal = canal.CanalMono()
        w = self.__call__(duree, maximum)
        s = self.alea.uniform(ecartmin, ecartmax, n)
        for i in range(n):            
            kanal.addT(pt, w)
            
            pt += s[i]
        return kanal.ex()

    def rafale(self, n, duree, maximum,
                        ecartmin=0.01, ecartmax=0.15):        
        """
        produit une rafale faite d'impulsions différentes
        (w est recalculé n fois)
        """
        pt = 0.0
        kanal = canal.CanalMono()
        s = self.alea.uniform(ecartmin, ecartmax, n)        
        for i in range(n):
            w = self(duree, maximum)
            kanal.addT(pt, w)
            
            pt += s[i]
        return kanal.ex()


