#-*-coding:utf-8-*-

"""
fournit la classe TissuHarmonique
"""


from samplerate import sr
import profils as _p
import ondes as _o
import canal08 as canal
import serie
import sys

__author__ = "Copyright (c) 2005-07 René Bastian (Wissembourg, FRANCE)"
__date__ = "2007.10.28"




class Tissu:
    def __init__(self, 
                 reverbduree = 6.0,
                 reverbpremiercoeff = 0.8,
                 couple = (9785,2348)):
        """
        'rapport : rapport intervallique utilisé pour calculer les
          fréquences.
        'reverbduree, reverbpremiercoeff : params pour la réverb
        Faire une méthode oÃ¹ les fréquences varient ...
        """         
        self.reverb = canal.Reverberation()
        self.reverb.duree = reverbduree
        self.reverb.premiercoeff = reverbpremiercoeff
        self.alea = serie.Serie002(*couple)

    def __call__(self, duree, dureeTotale, Freqs, Env, densite = 0.15):
        """
        'duree : durée du son préliminaire
        'dureeTotale : durée du son final
        'freq0, freq1 : fréquence la plus basse, fréquence la plus haute
        'Env : enveloppe du son préliminaire
          cette Env pourrait Ãªtre implicite ?
        'densite : dureeTotale / nombre de fois que le son
          préliminaire est inséré
        ampli ?, pano ? : on les laisse faire Ã  l'usage        
        """
        format = "%5.2f %9.2f %9.2f ... %9.2f %5.2f"
        print format % (duree, dureeTotale, Freqs[0], Freqs[-1], densite)
        ndirect = int(sr * duree)
        env = _p.ProfilBase(Env)
        c = canal.CanalMono(duree)
        for freq in Freqs:
           w = _o.sinusrecursif(freq, duree)
           w *= env.samples(len(w))
           c.addT(0.0, w)
           
        wr = self.reverb(c.a)
        
        traine = wr[ndirect:]        
        E = [[_p.cubica, duree/2, 0., 0., 1., 0.],
             [_p.cubica, duree/2, 1., 0., 0., 0.]]
        env = _p.ProfilBase(E)
        wt = traine * env.samples(len(traine))
        N = int(dureeTotale / densite) 
        cr = canal.CanalMono(dureeTotale)
        
        pt = 0.0        
        for i in range(N):
            cr.addT(pt, wt)
            pt = self.alea.uniform(0., dureeTotale)
        return cr.a
    
class TissuIntervallesFixes(Tissu):
    """
    construit d'abord une superposition d'oscillations espacées
    d'un intervalle unique : le son préliminaire ;
    ce tissu est réverbéré. On prend
    la traîne du résultat qui est répartie sériellement sur
    la durée totale voulue.
    """
    def __init__(self, rapport,
                 reverbduree = 6.0,
                 reverbpremiercoeff = 0.8,
                 couple = (9785,2348)):
        """
        'rapport : rapport intervallique utilisé pour calculer les
          fréquences.
        'reverbduree, reverbpremiercoeff : params pour la réverb
        Faire une méthode oÃ¹ les fréquences varient ...
        """ 
        self.r = rapport
        self.reverb = canal.Reverberation()
        self.reverb.duree = reverbduree
        self.reverb.premiercoeff = reverbpremiercoeff
        self.alea = serie.Serie002(*couple)
        
    def __call__(self, duree, dureeTotale, freq0, freq1, Env, densite = 0.15):
        """
        'duree : durée du son préliminaire
        'dureeTotale : durée du son final
        'freq0, freq1 : fréquence la plus basse, fréquence la plus haute
        'Env : enveloppe du son préliminaire
          cette Env pourrait Ãªtre implicite ?
        'densite : dureeTotale / nombre de fois que le son
          préliminaire est inséré
        ampli ?, pano ? : non il faut les laisser Ã  l'usage        
        """
        format = "%5.2f %9.2f %9.2f %9.2f %5.2f"
        print format % (duree, dureeTotale, freq0, freq1, densite)
        ndirect = int(sr * duree)
        env = _p.ProfilBase(Env)
        c = canal.CanalMono(duree)
        freq = freq0
        while freq < freq1:
           w = _o.sinusrecursif(freq, duree)
           w *= env.samples(len(w))
           c.addT(0.0, w)
           
           freq = freq * self.r
        wr = self.reverb(c.a)
        
        traine = wr[ndirect:]        
        E = [[_p.cubica, duree/2, 0., 0., 1., 0.],
             [_p.cubica, duree/2, 1., 0., 0., 0.]]
        env = _p.ProfilBase(E)
        wt = traine * env.samples(len(traine))
        N = int(dureeTotale / densite) 
        cr = canal.CanalMono(dureeTotale)
        
        pt = 0.0
        print N, duree
        for i in range(N):
            cr.addT(pt, wt)
            pt = self.alea.uniform(0., dureeTotale)
        return cr.a

class TissuHarmonique(object):
    """
    appels suggérés :
    rv = TissuHarmonique(freqh1, freqmax, Env, alea)
    w = rv.reverb(dureeTotale, durees, densite)
    ou
    w = TissuHarmonique(freqh1, freqmax, Env, alea).reverb(dureeTotale,
                                                           durees, densite)
    """
    
    def __init__(self, freqh1, freqmax, listeEnv, alea):
        """
        'freqh1
        'freqmax
        'Env
        'alea
        """
        self.freqh1, self.freqmax = freqh1, freqmax
        self.alea = alea
        self.kmax = int(freqmax / freqh1)
        self.envBase = _p.ProfilBase(listeEnv)
        self.reverb = canal.Reverberation()
        self.reverb.duree = 6.0
        self.reverb.premiercoeff = 0.8
        
    def __call__(self, dureeTotale, durees, densite):
        """
        'dureeTotale
        'durees : (dureemin, dureemax)
        'densite
        """
        N = int(dureeTotale * densite)
        
        c = canal.CanalMono(dureeTotale)
        for i in range(N):
            try:
                k = self.alea.randint(1, self.kmax)
            except ZeroDivisionError:
                k = 1
            freq = k * self.freqh1
            duree = self.alea.uniform(*durees)
            ampli = 1./k
            ampli = self.alea.uniform(0., 2.)
            w = _o.sinusrecursif(freq, duree)
            e = self.envBase.samples(len(w))
            w = w * e * ampli
            pt = self.alea.uniform(0.0, dureeTotale)
            c.addT(pt, w)
        return c.ex()

    def reverb(self, dureeTotale, durees, densite, revpremiercoeff = 0.8,
               revduree = 6.):
        self.rev = canal.Reverberation(dureeTotale)
        self.rev.premiercoeff = revpremiercoeff
        self.rev.duree = revduree        
        w = self(dureeTotale, durees, densite)
        return self.rev(w)


