#-*-coding:utf-8-*-

"""fournit la classe Serie002 : un générateur
sériel de nombres (très pseudo-)aléatoires ;
avantage sur random : ne sera pas « amélioré »
donc les séries resteront identiques.
"""

import types
import numpy as NY

__date__ = "2005.01.02, 06.02.07, 06.04.17, 08.08.19, 09.01.12"
__author__ = "Copyright (c) René Bastian 2005-2009 <rbastian@free.fr>"


from utiles import sr
import sys
import copy

class Serie001(object):
    """classe de série (presque) infinie de nombres entiers ;
    on peut en dériver des séries de nombres réels."""
    def __init__(self, x = 9, a = 11, m = 2**31-1, b = 0, decalage = False):
        """pour créer une instance de
        générateur de nombres pseudo-aléatoires.
        """
        
        self.x = x
        self.a = a
        self.b = b
        self.m = m
        self.i = 0
        self.xinit = x
        self.ncycle = 0
        self.decalage = decalage

    def seq(self):
        """
        calcule et fournit l'entier suivant.
        """
        
        self.x = (self.x*self.a+self.b) % self.m
        self.i += 1
        try:
            if self.x == self.xinit:
                print "\nFin de cycle"
                
                self.ncycle = self.i
                
                print self
                self.i = 0
                q = raw_input("Continuer ? ")
                if q in ["n", "N"]:
                    sys.exit()
                if self.decalage :
                    self.a += 1
                    self.xinit = self.x
                    print "Serie001 (nouveaux params) : ",
                    self.affiche()
            return self.x
        except RuntimeError:
            print self.x, self.xinit

    def seq2(self):
        "retourne la valeur suivante de  'x'"
        self.x = (self.x*self.a + self.b)*self.x % self.m
        self.i += 1
        try:
            if self.x == self.xinit:
                print "\nFin de cycle"
                
                self.ncycle = self.i
                self.affiche()            
                self.i = 0
                q = raw_input("Continuer ? ")
                if q in ["n", "N"]:
                    sys.exit()
                if self.decalage :
                    self.a += 1
                    self.xinit = self.x
                    print "Serie001 (nouveaux params) : ",
                    self.affiche()
            return self.x
        except RuntimeError:
            print self.x, self.xinit
    
    def seed(self, x, a):
        "introduit d'autres valeurs initiales"
        self.x = x
        self.a = a
        self.xinit = x
        self.ncycle = 0

    def couple(self):
        "retourne 'x' et 'a'"
        return self.x, self.a

    def quartuple(self):
        "retourne le quatuor des valeurs"
        return self.x, self.a, self.m, self.b
        
    def __str__(self, tout=True):
        "affiche l'état actuel"
        if tout:
            return self.__class__.__name__  + str(self.__dict__)
        else:
            format = "x: %10i a: %10i m: %10i b: %10i" 
            return format % (self.x, self.a, self.m, self.b)
                
    def ensemble(self, n):
        """
        fournit n nombres distincts.
        """
        R = []
        for _ in xrange(n):
            R.append(self.seq())
        E = set(R)
        
        return E
    
    def affiche(self):
        "affiche l'état actuel"
        forme = "a: %10i m: %10i i: %10i xinit: %10i ncycle: %10i x: %10i" 
        print forme % (self.a, self.m, self.i, self.xinit, self.ncycle, self.x)
        
    def courant(self):
        "affiche qqs attributs de l'état actuel"
        print "a: %i m: %i i: %i x: %i" % \
        (self.a, self.m, self.i, self.x)

class Serie002(Serie001):
    """classe de série (presque) infinie de nombres réels.
    L'initialisation selon 'Serie001' est obligatoire.
    """
    def __init__(self, x = 9, a = 11, m = 2**31-1, b = 0, decalage = False):
        "initialisation"
        Serie001.__init__(self, x, a, m, b, decalage)
        self.xT = None

    def seed(self, x, a):
        """
        change les valeurs initiales.
        """
        self.x = x
        self.a = a
        self.xinit = x
        self.ncycle = 0

    def __call__(self, a0, a1, N = 1):
        """
        retourne les 'N' prochaines valeurs réelles (float) entre a0 et a1.
        (identique à 'uniform')
        """
        if a1 < a0: 
            a1, a0 = a0, a1
        if N == 1 :
            x = self.fseq()            
            return (a1 - a0) * x + a0
        elif N > 1:            
            
            z = NY.zeros(N, NY.float64)
            for i in range(N):
                x = self.fseq()
                z[i] = (a1 - a0) * x + a0
            return z
        
    def entier(self, a0, a1, N=1):
        """retourne 'N' valeurs entre a0 et a1"""
        
        
        if a1 < a0: 
            a1, a0 = a0, a1
        if N == 1 :
            return (self.seq() % (abs(a1-a0))) + a0
        elif N > 1:
            z = [(self.seq() % (abs(a1-a0))) + a0 for _ in range(N)]
            return z
        
    
    def randint(self, a0, a1, N=1):
        """retourne une valeur entre a0 et a1"""
        
        
        if N == 1:
            return (self.seq() % (abs(a1-a0))) + a0
        else:
            return self.entier(a0, a1, N)
    
    def choice(self, listeE):
        """choisit une valeur de la liste E

        Cette méthode permet de guider les choix (cf SoleilBlanc/melos_1):
        au lieu de calculer un longue série de valeurs on peut
        faire choisir soit dans une liste assez courte (donc un choix
        quasi modal) ou très longues (donc un choix quasi sériel.
        """
        p = self.entier(0, len(listeE))
        return listeE[p]

    def choix(self, E, n):
        """tire n éléments de la liste E

        Emploi: voir 'choice'
        """
        if n == len(E):
            return E
        elif n < len(E):
            R = []
            while len(R) < n:
                x = self.choice(E)
                if not x in R:
                    R.append(x)
            return R
        else:
            print "Serie002.choix: len(E) < n"
            return None
                
    
    def fseq(self):
        """retourne la prochaine valeur réelle."""
        _ = self.seq()
        return float(self.x)/self.m
    
    def uniform(self, a0, a1, N = 1):
        """retourne les N prochaines valeurs réelles (float) entre a0 et a1."""
        if a1 < a0: 
            a1, a0 = a0, a1
        if N == 1 :
            x = self.fseq()
            return (a1 - a0) * x + a0
        elif N > 1:
            z = NY.zeros(N, NY.float64)
            for i in range(N):
                x = self.fseq()
                z[i] = (a1 - a0) * x + a0
            return z
        
    def pt(self, a1):
        "multiplie le nombre suivant par 'a1' - boff..."
        x = self.fseq()
        return a1*x
    
    def vunique(self, n, duree = 1.0):
        "retourne un vecteur de 'n' nombres dont la somme est 'duree'"
        V = []
        for _ in range(int(n)):
            V.append(self.pt(duree))
        s = NY.sum(V)
        V = NY.array(V)/s        
        return V * duree
    
    def vecteur(self, n, duree):
        """produit un vecteur de n points temporels sur duree."""
        self.xT = []
        for _ in range(int(n)):
            self.xT.append(self.pt(duree))
        self.xT.sort()
        return self.xT
    
    def distribution(self, xmin, xmax, nn):
        """produit un vecteur de nn nombres réels entre xmin et xmax."""
        T = []
        for _ in range(nn):
            T.append(self.uniform(xmin, xmax))
        return NY.asarray(T)
    
    def bruit(self, n, xmin = -1.0, xmax = +1.0):
        "presque comme 'noise' ;-)"
        return self.uniform(xmin, xmax, n)

    def menge(self, n):
        """
        retourne une suite de nombres de 0 à n-1
        sans répétition.
        """
        R = []
        while len(R) < n:
            x = self.randint(0, n)
            if not x in R:
                R.append(int(x))
        return R

    def noise(self, duree, xmin = -1.0, xmax = +1.0):
        "du bruit entre -1 et +1"
        n = int(sr*duree)
        return self.uniform(xmin, xmax, n)
    
    def bruit_decroissant(self, n, amorti):
        "du bruit amorti"
        if amorti == 1.0:
            return self.bruit(n)
        elif amorti > 1.0:
            return None
        else:
            b = NY.ones(n, NY.float64)
            for i in range(1, n):
                b[i] = b[i-1]*amorti
            for i in range(n):
                b[i] *= self.uniform(-1., +1.)
        return b

    def impulsions(self, duree, densite):
        """
        fournit une suite d'impulsions
        """
        n = int(duree*sr)
        p = int(n*densite)
        w = NY.zeros(n, NY.float64)
        
        for _ in range(p):
            ix = self.randint(0, n)            
            w[ix] = 1.0
        return w
    
    def suite(self, E):
        """
        produit une permutation aleatoire des elements de E
        """
        if not type(E) == types.ListType:
            E = E.tolist()
        R = []
        for _ in range(len(E)):
            indice = self.entier(0, len(E))
            R.append(E[indice])
            del E[indice]
        return R

    def suiteI(self, n, nmin, nmax):
        """retourne une suite de n nombres entre nmin et nmax,
        nmax exclu."""
        R = []
        for _ in range(n):
            R.append(self.entier(nmin, nmax))
        return R

    def suiteE(self, n, E):
        """
        retourne une suite sérielle de 'n' éléments
        de la liste 'E'.
        """
        R = []
        p = len(E)
        for _ in range(n):
            ix = self.entier(0, p)
            R.append(E[ix])
        return R

    def promenade(self, n, ixini, E):
        """
        le promeneur part de 'ixini' pour faire 'n' pas
        de grandeurs donnés de 'E'.
        'n :nombre de pas
        'ixini : indice de départ
        'ecarts :liste d'indices p.ex. [-1, 1] ou [-2, 0, 1, 1]
        qui détermineront la promenade.
        """
        R = NY.zeros(n, NY.int32)
        R[0] = ixini
        p = len(E)
        for i in range(1, n):
            R[i] = R[i-1] + E[self.entier(0, p)]
        return R

    def dissemination(self, n, dimension):
        """produit 'n' fois la valeur 1. 
        sur un vecteur de 'dimension' zéros"""
        v = NY.zeros(dimension, NY.float64)
        for _ in range(n):
            ix = self.randint(0, dimension)
            v[ix] = 1.0
        return v
    
    def insertion(self, nombre, xmin = 0.0, xmax = 1.0):
        """produit un vecteur entre 'xmin' et 'xmax' 
        ces deux valeurs incluses."""
        
        v = NY.array([xmin] + list(self.uniform(xmin, xmax, nombre-2)) + [xmax])
        v.sort()
        return v

    def close(self):
        "on ferme ..."
        return self.x, self.a, self.b



