#-*-coding:utf-8-*-

"""
but: faire une synthèse de Fourier par
harmoniques et enveloppes.
"""

from origine import Origine0
from conversions08 import prony2freq
from canal import CanalMono
from profilbase import ProfilBase
from sinus08 import sinusrecursif

__date__ = "2009.02.21"
__author__ = "Copyright (c) 2009 René Bastian (Wissembourg, FRANCE)"

class Fourier(Origine0):
    """
    classe pour une synthèse de Fourier élémentaire.  
    """
    def __init__(self, enveloppesH):
        """
        crée l'instance avec les enveloppes
        'enveloppesH' : liste de tuples (rang, duree, profilA, ampli, ecart)
        'enveloppesH' : liste de couples (rang, duree, profilA, 
                                ampli, ecart, profilPano)
        """
        Origine0.__init__(self)
        self.envH = enveloppesH

    def mono(self, hauteur):
        """
        'hauteur' : hauteur en prony
        """
        freq0 = prony2freq(hauteur)
        c = CanalMono()
        for p in self.envH:
            if len(p) == 4:
                rang, duree, envP, ampli = p
                ecart = 0.0
            elif len(p) == 5:
                rang, duree, envP, ampli, ecart = p
            freq = freq0 * rang
            w = sinusrecursif(freq, duree)
            env = ProfilBase(envP).samples(len(w))
            w *= env
            w *= ampli
            c.addT(ecart, w)
        return c.ex()

    def pano(self, hauteur):
        """
        'hauteur' : hauteur en prony
        """
        freq0 = prony2freq(hauteur)
        cg = CanalMono()
        cd = CanalMono()
        for p in self.envH:
            if len(p) == 5:
                rang, duree, envP, ampli, envPano = p
                ecart = 0.0
            elif len(p) == 6:
                rang, duree, envP, ampli, envPano, ecart = p
            freq = freq0 * rang
            w = sinusrecursif(freq, duree)
            nw = len(w)
            env = ProfilBase(envP).samples(nw)
            w *= env
            w *= ampli
            if isinstance(envPano, list):
                pano =  ProfilBase(envPano).samples(nw)
            wg, wd = w * pano, w * (1. - pano)
            cg.addT(ecart, wg)
            cd.addT(ecart, wd)
            freq += freq0
        return cg.ex(), cd.ex()



if __name__ == "__main__":
    from profils import recta
    import pprint
    import live
    
    audio = live.Audio()
    pp = pprint.PrettyPrinter(indent=4)
    
    harms = [
        ( 1, 15.0, [[recta, 0.1, 0., 1.], [recta, 0.9, 1., 0.]],  5., 0.0),
        ( 3, 15.0, [[recta, 0.3, 0., 1.], [recta, 0.7, 1., 0.]],  8., 1.0),
        ( 5, 15.0, [[recta, 0.5, 0., 1.], [recta, 0.5, 1., 0.]], 13., 2.0),
        (11, 15.0, [[recta, 0.1, 0., 1.], [recta, 0.9, 1., 0.]],  5., 3.0),
        (13, 15.0, [[recta, 0.1, 0., 1.], [recta, 0.9, 1., 0.]],  8., 4.0)
        ]
    fr = Fourier(harms)
    print fr
    w = fr.mono(69.0)
    audio.play(w, w)
    audio.close()


