#-*-coding:utf-8-*-


"""
fourit la classe Craques: bruits rugueux
"""

import utiles as _u
import profils as _p
import filtres as _f
import decibel
import canal08
import serie
from origine import Origine

__author__ = "Copyright (c) 2005-09 René Bastian (Wissembourg, FRANCE)"
__date__ = "2007.03.22, 09.01.29"

class Craques(Origine):
    """
    'Craques' synthétise un son par combinaison de petits grains de sons,
    """
    def __init__(self, envE = None, deltaprony=0.2, couple=(8861, 6637)):
        """
        'envE' : enveloppe du craque
        'deltaprony' : finesse du filtre
        'alea' : un générateur sériel
        """
        Origine.__init__(self)
        if not envE:
            envE = [[_p.recta, 0.005, 0.0, 1.0, 0.5],
                 [_p.recta, 0.005, 1.0, 0.0, 2.0]]
        self.envE = _p.ProfilBase(envE)
        self.alea = serie.Serie002(*couple)
        self.u = decibel.Decibel()
        self.deltaprony = deltaprony
        self.dbinf, self.dbsup = 3.0, 10.0


    def __call__(self, duree, ngrains, pronyinf, pronysup, dureegrain=0.01):
        """
        suite de filtres PB statiques
        'duree' : durée totale du groupe de son produits (2 sec)
        'ngrains' : nombre de grains (4 à 8 ?)
        'deltaprony' : largeur de la bande de bruit
        'pronyinf' : limite inférieure de la hauteur
        'pronysup' : limite supérieure de la hauteur
        """
        c = canal08.CanalMono()
        
        e = self.envE.temps(dureegrain)
        x = self.alea.uniform(-1, +1, len(e))
        grain = e * x
        
        for _ in range(ngrains):
            amplidb = self.alea.uniform(self.dbinf, self.dbsup) 
            ampli = self.u.db2x(amplidb)
            g = grain * ampli
            pronycoupure = self.alea.uniform(pronyinf, pronysup)
            gf = _f.filtreBPrecursif(g, pronycoupure, self.deltaprony) 
            
            pt = self.alea.uniform(0.0, duree)
            c.addT(pt, gf)
            
        return c.a / _u.extremum(c.a)

    def evt(self, duree, ngrains, varProny, deltaProny, dureeGrain):
        """
        'varProny' : profil de la variation de la hauteur centrale
        du filtre,
        'deltaProny' : profil de la variation de la largeur de bande
        du filtre,
        'dureeGrain' : profil de la variation de la durée du grain.
        
        La répartition sur la durée n'est plus sérielle, mais
        à espacement régulier.
        """
        pt, dt = 0.0, duree/float(ngrains)                    
        c = canal08.CanalMono()
        pronycoupure = _p.ProfilBase(varProny).samples(ngrains)
        deltaprony = _p.ProfilBase(deltaProny).samples(ngrains)
        dureegrain = _p.ProfilBase(dureeGrain).samples(ngrains)
        for i in range(ngrains):
            amplidb = self.alea.uniform(self.dbinf, self.dbsup) 
            ampli = self.u.db2x(amplidb)
            e = self.envE.temps(dureegrain[i])
            x = self.alea.uniform(-1, +1, len(e))
            grain = e * x
            g = grain*ampli            
            gf = _f.filtreBPrecursif(g, pronycoupure[i],
                                     deltaprony[i]) 
            c.addT(pt, gf)
            pt += dt            
        return c.a / _u.extremum(c.a)
            


