#-*-coding:utf-8-*-

"""
contient les classes ProfilBasePlus, ProfilBaseMicro
"""

import types
from samplerate import sr
from conversions08 import extremum
from profils import recta, cubica, profil 
import profils as _p
import ajuster as _aj
import serie
from origine import Origine

__author__ = "Copyright (c) 2005-09 René Bastian (Wissembourg, FRANCE)"
__date__ = "2007.06.14, 07.10.06, 09.02.04"


class ProfilBasePlus(Origine):
    """
    superpose un micro-profil au ProfilBase ;
    le micro-profil ne varie pas homothétiquement.
    À chaque appel, la classe calcule un micro-profil.

    Étendre cette classe en faisant générer un profil
    avec des 'dt' et de 'y' variant sériellement.
    """
    def __init__(self, P, courbures=None, couple=(91143, 34119)):
        """
        'P' : le profil de base principal
        'courbures' : courbures appliquées dans le calcul des recta
        'cdt' : limites de la variation de dt (fournis lors de l'appel)
        'couple' : générateur sériel
        """
        Origine.__init__(self)
        self.pb = _p.ProfilBase(P) 
        self.alea = serie.Serie002(couple[0], couple[1])
        self.courbs = courbures
        self.genres = [recta, cubica]
        self.profilliste = []
        self.dtmin, self.dtmax = None, None

    def _micro(self, duree):
        """
        calcule le micro-profil pour une enveloppe (ne pas appliquer
        pour une mod.freq); 
        pourquoi ?
        """
        t = 0.0
        T = []
        while t <= duree:
            dt = self.alea.uniform(self.dtmin, self.dtmax)
            if t + dt <= duree:
                T.append(dt)
            else:
                T.append(duree - sum(T))
            t += dt
        M = []
        y0 = 0.0
        for dt in T:
            y1 = self.alea.uniform(0.0, 1.0)
            genre = self.alea.choice(self.genres)
            if genre == recta:
                try:
                    courbure = self.alea.uniform(self.courbs[0], self.courbs[1])
                except ValueError:
                    courbure = 1.0
                M.append([recta, dt, y0, y1, courbure])
            elif genre == cubica:
                M.append([cubica, dt, y0, 0., y1, 0.])
            t += dt
            y0 = y1
        
        
        M[-1][-2] = 0.0
        
        self.profilliste = M
        p = profil(M)
        return p / extremum(p)

    def samples(self, n, rapport, cdt):
        """
        'n' : nombre d'échantillons
        'rapport' : en principe << 1.0, mais
        on obtient des profils tortueux si 0. < rapport < 1;
        'rapport' peut être un profil-liste pour évoluer dans le temps.
        'cdt' : (durée min, durée max) du fragment de profil
        """
        self.dtmin, self.dtmax = cdt
        px = self.pb.samples(n)
        duree = float(n)/sr
        if isinstance(rapport, list):
            rapport = _p.ProfilBase(rapport).samples(n)
        micro = self._micro(duree)* rapport
        px, micro = _aj.prolongerN(px, micro)
        px *= (1. + micro)
        return px 
    
    def temps(self, duree, rapport, cdt):
        """
        'duree' : durée du profil
        'rapport' : en principe << 1.0, mais
        on obtient des profils tortueux si 0. < rapport < 1.
        'rapport' peut être un profil-liste pour évoluer dans le temps.
        'cdt' : (durée min, durée max) du fragment de profil
        """
        self.dtmin, self.dtmax = cdt
        px = self.pb.temps(duree)
        
        if isinstance(rapport, list):
            n = len(px)
            rapport = _p.ProfilBase(rapport).samples(n)
        micro = self._micro(duree)* rapport        
        px, micro = _aj.prolongerN(px, micro)
        px *= (1. + micro)
        return px 


