#-*-coding:utf-8-*-

"""fournit des profils"""

import copy


import numpy as NY
from samplerate import sr
from conversions08 import extremum
import ajuster as _a

__author__ = "Copyright (c) René Bastian <rbastian@free.fr>"
__date__ = "2003.03.03, 2006.02.10, 2007.02.11, 07.04.22"

def profil(listeP):
    """
    calcule un vecteur-profil à partir de sa description.
    'P' : liste des composants du profil
    06.12.07 ; env. 12 % plus rapide que la précédente version.
    La version 'liste de listes' a l'avantage de pouvoir être
    utilisée par 'ProfilBase'.
    """
    if listeP == []:
        print "profil(listeP) == []"
        return NY.zeros(1)
    listeR = []
    if not isinstance(listeP, list):
        print "profil:listeP", type(listeP)
        return
    for e in listeP:
            listeR.append(e[0](e[1:]))
            
        
    
    
    
    
    try:
        return NY.concatenate(listeR)
    except ValueError:
        print "profils.profil:ValueError"
        print "listeP", listeP
        print "listeR", listeR
        for r in listeR:
            if not r:
                print "Nada"
            else:
                print str(r)
def rectaf(D):
    "comme recta"
    return recta(D)

def curvaf(D):
    "comme curva"
    return curva(D)

def cubicaf(D):
    "comme cubica"
    return  cubica(D)

def recta(D):
    """
    trace une droite 
    syntaxes de D: 
        [duree, y_initial, y_final]
        [duree, y_initial, y_final, courbure]
    """
    try:
        duree, y0, y1, puiss = D
    except ValueError:
        duree, y0, y1 = D
        puiss = 1.0
    nd = int(duree * sr)
    if nd <= 1:
        return NY.asarray([(y0 + y1)/2])
    if y0 == y1:
        return NY.ones(nd, NY.float64) * y0
    if puiss != 1.0:
        u = NY.arange(0.0, 1.0, 1.0/nd)
        u **= puiss
        u *= y1 - y0
        return y0 + u
    else:
        return y0 + NY.arange(0.0, 1.0, 1.0/nd) * (y1 - y0)

def curva(D):
    """trace une courbe exponentielle
        - allant de D[1] à D[2] en une durée D[0]
        - selon une courbure D[3]
        D[4] peut accentuer la courbure
    syntaxes:
           [duree, y_initial, y_final, courbure]
           [duree, y_initial, y_final, courbure, puissance]
    """
    duree = D[0]
    yi = D[1]
    yt = D[2]
    c = D[3] 
    
    try:
        coeff = D[4]
    except ValueError:
        coeff = 1.0
    a = NY.exp(NY.log(NY.abs(yt - yi)) - c * NY.log(duree))
    if yt < yi: 
        a = -a
    T = NY.arange(0.0, duree, 1.0/sr)
    if coeff == 1.0:
        return a * (T**c) + yi
    else:
        y = a * (T**c) + yi
        return y**coeff

def cubica(liste):
    """
    trace une courbe que les mathématiciens désignent par 'cubique'.
    Syntaxes:
        liste = [dt, y0, p0, y1, p1 ]
        liste = [dt, y0, p0, y1, p1, puissance]
    L'usage de 'puissance' est malaisé car elle change les pentes p0 et p1.
    """    
    try:
        duree, y0, p0, y1, p1, puiss = liste
    except ValueError:
        
        duree, y0, p0, y1, p1 = liste
        return courbeCubique08(0.0, y0, p0, duree, y1, p1)        
    else:
        
        nd = int(duree * sr)
        courbure = NY.arange(0., 1., 1./nd)**puiss
        y = courbeCubique(0.0, 0.0, p0, duree, y1 - y0, p1)
        
        y, courbure = _a.prolongerZ(y, courbure)
        return y * courbure + y0


def courbeCubique08(x0, y0, p0, x1, y1, p1):
    """retourne une cubique    
    'x0, y0 : coordonnée du point initial
    'p0 : tangente au point initial
    'x1, y1 : coordonnée du point final
    'p1 : tangente au point final    
    """
    R = coeffscubiquex(x0, y0, p0, x1, y1, p1)
    T = NY.arange(x0, x1, 1.0/sr)
    S = a_valeurCubique(R, T)
    return S

def courbeCubique(x0, y0, p0, x1, y1, p1):
    """retourne une cubique    
    'x0, y0 : coordonnée du point initial
    'p0 : tangente au point initial
    'x1, y1 : coordonnée du point final
    'p1 : tangente au point final    
    """
    R = coeffscubiquex(x0, y0, p0, x1, y1, p1)
    T = NY.arange(x0, x1, 1.0/sr)
    S = a_valeurCubique(R, T)
    return S

def coeffscubiquex(x0, y0, p0, x1, y1, p1):
    """retourne les coeffs a, b, c, d
    tels que a * x**3 + b * x**2 + c * x + d
    passe par (x0, y0, p0, x1, y1, p1)
    """
    M = [[x0**3,   x0**2, x0, 1],
         [x1**3,   x1**2, x1, 1],
         [3 * x0**2, 2 * x0,  1,  0],
         [3 * x1**2, 2 * x1,  1,  0]]
    Y = [y0, y1, p0, p1]
    R = systemeCramer(M, Y)
    return R

def a_valeurCubique(F, x):
    """    
    est plus rapide que valeurCubique
    """
    return (((F[0] * x + F[1]) * x + F[2]) * x + F[3])

            
def reduire(M, ix):
    """réduction du tableau M, c-à-d
    barrer la ligne d'indice ix
    et toutes les valeurs de la colonne d'indice 0
    """
    
    R = copy.deepcopy(M)
    del R[ix]
    for i in range(len(R)):
        del R[i][0]
    return R

def reduire08(M, ix):
    """réduction du tableau M, c-à-d
    barrer la ligne d'indice ix
    et toutes les valeurs de la colonne d'indice 0
    """
    
    R = copy.deepcopy(M)
    del R[ix]
    for i in range(len(R)):
        del R[i][0]
    return R

def carre(M):
    """contrôle si le tableau est un carré
    """
    x = 1
    for i in range(len(M)):
        if len(M[i]) != len(M):
            x = 0
    return x

def determinant(M):
    """retourne déterminant du tableau M
    """
    if not carre(M):
        print "Ce n'est pas un tableau carré -> pas de déterminant."
        return None
    elif len(M) == 1:
        return M[0]
    elif len(M) == 2:
        m = M[0][0] * M[1][1] - M[0][1] * M[1][0]
        return m
    else :
        m = 0.0
        signe = 1.0
        for i in range(len(M)):
            R = reduire(M, i)
            print type(R), type(determinant(R))
            m = m + M[i][0] * determinant(R) * signe
            signe = -signe
    return m

def determinant08(M):
    """retourne déterminant du tableau M
    """
    return NY.linalg.det(M)

def systemeCramer(M, Y, affiche = 0):
    """
    retourne R tel que M * R = Y
    """
    
    k = determinant08(M)
    if k != 0:
        k = 1.0/k
    else :
        print "systemeCramer: Il n'y a pas de solution"
        return None    
    R = []
    for i in range(len(M)):
        H = copy.deepcopy(M)
        for j in range(len(Y)):
            H[j][i] = Y[j]
        if affiche:
            print H
        
        r = k * determinant08(H)
        R.append(r)
    return R



def profiltuple(listeP, alea, liste=True): 
    """
    La liste-profil listeP contient des données tels que (y0, y1) ou (dt0, dt1)
    La valeur effective y ou dt sera déterminée sériellement à l'aide
    de 'alea'.

    L'option 'liste' permet de choisir un retour en profil calculé
    ou en profil-liste.
    SIMPLIFIABLE ? car on peut écrire alea(a, b) ?
    """
    listeR = []
    for p in listeP:
        r = []
        for pp in p:
            if (type(pp) == types.TupleType) and (len(pp) == 2):
                r.append(alea.uniform(pp[0], pp[1]))
            elif (type(pp) == types.TupleType) \
                     and (type(pp[0]) == types.TupleType) \
                     and (len(pp[0]) == 2):
                x = alea.uniform(pp[0][0], pp[0][1])
                r.append((x))
            else:
                r.append(pp)
        listeR.append(r)
    if liste:
        return listeR
    else:
        return profil(listeR)


